/* StarDork 0.6 -by Ryan Kulla (gt3) ambiod@sbcglobal.net Jan 2003 */
#include <curses.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>
#include <sys/ioctl.h>
#include <sys/kd.h>


#define XMIN 1
#define YMIN 1
#define LEVEL_MAX 34 /* how many levels to have */
#define sndon(freq) ioctl(0, KIOCSOUND, 1193810 / freq)
#define sndoff() ioctl(0, KIOCSOUND, 0)


struct dork_ship {
    char *ship;
    char *star;
    char *goal;
    int power;
    int lives; 
    int level;
    int moves;
    int plant1[255];
    int plant2[255];
    int obj_y;
    int obj_x;
} space = { "X", ".", "@" };


void move_ship(int, int);
void shoot_right(int, int);
void shoot_up(int, int);
void shoot_down(int, int);
void shoot_left(int, int);
void shoot_d_up_left(int, int);
void shoot_d_up_right(int, int);
void shoot_d_down_left(int, int);
void shoot_d_down_right(int, int);
void check_if_bumped(int, int);
void check_if_goal_is_hit(int, int);
void check_if_you_won(void);
void rand_plant(void);
void check_if_dead(int y, int x);
void set_stuff_up(void);
void die(int y, int x);
int check_bounds_y(int);
int check_bounds_x(int);
void show_stars(int, int);
void big_deal(void);
void count_moves(void);
static void catch_sigint(int signo);


struct dork_ship store[255];
int i, j, b = 0, fire_speed = 1;
char *blank = " ";


int main(int argc, char **argv)
{
int y, x;
struct sigaction sa_old, sa_new;

    sa_new.sa_handler = catch_sigint;
    sigemptyset(&sa_new.sa_mask);
    sa_new.sa_flags = 0;
    sigaction(SIGINT, &sa_new, &sa_old);
    
    initscr(); 
    curs_set(0);
    cbreak(); 
    noecho(); 
    keypad(stdscr, TRUE); 
    scrollok(stdscr, TRUE);

    space.obj_y = rand() % (LINES - 2);
    space.obj_x = rand() % (COLS - 2) - 1;
    space.power = 9;
    space.level = 1;
    space.lives = 3;	
    space.moves = 0;
    y = 3, x = 15; /* Where to initially place the spaceship */

    rand_plant(); 
    attrset(A_BOLD);
    mvprintw(y, x, space.ship); 
    attrset(A_NORMAL);
    move_ship(y, x);

    refresh(); 
    endwin(); 
    return(0);
}


void move_ship(int y, int x)
{
    show_stars(y, x);
    while (1) {
        set_stuff_up();
        attrset(A_BOLD);
        switch (getch()) {
            case KEY_UP: 
                count_moves();
                if ((check_bounds_y(y)) == 1) 
                    break; 
                mvprintw(y, x, blank);
                mvprintw(--y, x, space.ship);          
                break;
            case KEY_DOWN: 
                count_moves();
                if ((check_bounds_y(y)) == 2)
                    break;
                mvprintw(y, x, blank);
                mvprintw(++y, x, space.ship);
		break;
            case KEY_RIGHT:
                count_moves();
                if ((check_bounds_x(x)) == 2) 
                    break;
                mvprintw(y, x, blank); 
                mvprintw(y, ++x, space.ship);
                break;
            case KEY_LEFT:
                count_moves();
                if ((check_bounds_x(x)) == 1) 
                    break;
                mvprintw(y, x, blank); 
                mvprintw(y, --x, space.ship);
                break;
            case KEY_HOME:
                count_moves();
                if ((check_bounds_y(y)) == 1) 
                    break;
                if ((check_bounds_x(x)) == 1) 
                    break;
                mvprintw(y, x, blank); 
                mvprintw(--y, --x, space.ship);
                break;
            case KEY_PPAGE:
                count_moves();
                if ((check_bounds_y(y)) == 1) 
                    break;
                if ((check_bounds_x(x)) == 2) 	
                    break;
                mvprintw(y, x, blank); 
                mvprintw(--y, ++x, space.ship);
                break;
            case KEY_END:
                count_moves();
                if ((check_bounds_y(y)) == 2) 
                    break;
                if ((check_bounds_x(x)) == 1) 
                    break;
                mvprintw(y, x, blank); 
                mvprintw(++y, --x, space.ship);
                break;
            case KEY_NPAGE:
                count_moves();
                if ((check_bounds_y(y)) == 2) 
                    break;
                if ((check_bounds_x(x)) == 2) 	
                    break;
                mvprintw(y, x, blank); 
                mvprintw(++y, ++x, space.ship);
                break;
            case 'd': 
                shoot_left(y, x); 
                break;
            case 'a': 
                shoot_up(y, x); 
                break;
            case 's': 
                shoot_down(y, x); 
                break;
            case 'f': 
                shoot_right(y, x); 
                break;
            case 'z': 
                shoot_d_up_left(y, x); 
                break;
            case 'x': 
                shoot_d_up_right(y, x); 
                break;
            case 'c': 
                shoot_d_down_left(y, x); 
                break;
            case 'v': 
                shoot_d_down_right(y, x); 
                break;
            }
            check_if_bumped(y, x);
            check_if_goal_is_hit(y, x); 
        } 
}


void set_stuff_up(void)
{
    attrset(A_BOLD);
    for (i = 1; i <= space.power; i++)
        mvaddch(0, i + 17, '|');

    mvprintw(0, i + 17, blank);
    mvprintw(0, 11, "Power: "); 
    mvprintw(0, 30, "Lives: %d", space.lives);
    mvprintw(0, 40, "Level: %d", space.level);
    attrset(A_BLINK | A_BOLD);
    mvprintw(space.obj_y, space.obj_x, space.goal);
    attrset(A_NORMAL);
}


void count_moves(void) 
{
    space.moves++;
    mvprintw(0, 50, "Moves: %d", space.moves);
}


void shoot_right(int y, int x)
{
int old_y = y, old_x = x; 

    sndon(7000);
    for (i = x; i < (COLS - 2) - 1; i++) {
        mvprintw(y, ++x, "-");
        napms(fire_speed - 1);
        refresh();
        check_if_goal_is_hit(y, x);
        mvprintw(y, x, blank);
        sndoff();
    }
    show_stars(old_y, old_x);
}


void shoot_up(int y, int x)
{
int old_y = y, old_x = x;

    sndon(7000);
    for (i = y; i > 1; i--) {
        mvprintw(--y, x, "|"); 
        napms(fire_speed + 9); 
        refresh();
        check_if_goal_is_hit(y, x);
        mvprintw(y, x, blank);
        sndoff();
    }
    show_stars(old_y, old_x);
}


void shoot_down(int y, int x)
{
int old_y = y, old_x = x;

    sndon(7000);
    for (i = y; i < (LINES - 2); i++) {
        mvprintw(++y, x, "|");
        napms(fire_speed + 9); 
        refresh();
        check_if_goal_is_hit(y, x);
        mvprintw(y, x, blank);
        sndoff();
    }
    show_stars(old_y, old_x);
}


void shoot_left(int y, int x)
{
int old_y = y, old_x = x;

    sndon(7000);
    for (i = x; i > 1; i--) {
        mvprintw(y, --x, "-");
        napms(fire_speed - 1); 
        refresh();
        check_if_goal_is_hit(y, x);
        mvprintw(y, x, blank);
        sndoff();
    }
    show_stars(old_y, old_x);
}


void shoot_d_up_left(int y, int x)
{
int old_y = y, old_x = x;

    i = y;
    sndon(7000);
    while (i > 1) {
        x -= 2, i--;
        mvprintw(--y, x, "\\");
        napms(fire_speed); 
        refresh();
        check_if_goal_is_hit(y, x);
        mvprintw(y, x, blank);
        sndoff();
    }
    show_stars(old_y, old_x);
}


void shoot_d_up_right(int y, int x)
{
int old_y = y, old_x = x;

    i = y;
    sndon(7000);
    while (i > 1) {
        x += 2, i--;
        mvprintw(--y, x, "/");
        napms(fire_speed); 
        refresh();
        check_if_goal_is_hit(y, x);
        mvprintw(y, x, blank);
        sndoff();
     }
    show_stars(old_y, old_x);
}


void shoot_d_down_left(int y, int x)
{
int old_y = y, old_x = x;

    i = y;
    sndon(7000);
    while (i < (LINES - 2)) {
        x -= 2, i++;
        mvprintw(++y, x, "/");
        napms(fire_speed); 
        refresh();
        check_if_goal_is_hit(y, x);
        mvprintw(y, x, blank);
        sndoff();
    }
    show_stars(old_y, old_x);
}


void shoot_d_down_right(int y, int x)
{
int old_x = x, old_y = y;

    i = y;
    sndon(7000);
    while (i < (LINES - 2)) {
        x += 2, i++;
        mvprintw(++y, x, "\\");
        napms(fire_speed); 
        refresh();
        check_if_goal_is_hit(y, x);
        mvprintw(y, x, blank);
        sndoff();
    }
    show_stars(old_y, old_x);
}


void rand_plant(void)
{
int r1, r2; 

    for (i = 1; i < (COLS - 2); i++) {
        r1 = rand() % (LINES - 1);
        r2 = rand() % (COLS - 2); 
        store[b].plant1[i] = r1, store[b].plant2[i] = r2;
    }
    b++;
}


void show_stars(int y, int x)
{
int s;

    attrset(A_NORMAL);
    for (i = 0; i < (COLS - 2); i++) {
        for (s = 0; s < (COLS - 2); s++) 
            mvprintw(store[s].plant1[i], store[s].plant2[i], space.star);
    }
}


void check_if_goal_is_hit(int y, int x)
{
    if (y == space.obj_y && x == space.obj_x) {
        sndon(1000);
        space.obj_y = rand() % 24; 
        space.obj_x = rand() % (COLS - 2);
        space.level++;
        rand_plant(); 
        check_if_you_won();
        show_stars(y, x);
    }
}


void check_if_you_won(void)
{
    sndoff();
    if (space.level == LEVEL_MAX)
        big_deal(); 
}


void check_if_bumped(int y, int x)
{
int c;   
 
    for (i = 0; i < (COLS - 2); i++) {
        for (c = 0; c < (COLS - 2); c++) {
            if (y == store[c].plant1[i] && x == store[c].plant2[i]) {
                standout(); 
                mvprintw(y, x, "X"); 
                standend();
                sndon(65);
                space.power--;  
                sndoff();
            }
        }
    }
    check_if_dead(y, x);
}


void check_if_dead(int y, int x)
{
    if (space.power < 1) { 
        space.lives--;
        space.power = 9; 
	
        if (space.lives < 1)
            die(y, x);
    }
}


void die(int y, int x)
{
int i = 1, j = LINES, fr;

    attrset(A_BOLD); 
    mvprintw(y, x, space.ship); 
    getch(); 
    mvprintw(y, x, blank); 

    srand((unsigned)time(NULL));
    while (i < (LINES / 2 - 2)) {
        ++i;
        j--;
        fr = (rand() % 150) + 100;
        sndon(fr);
        mvprintw(i, (COLS / 2 - 30), blank);
        mvprintw(i + 1, (COLS / 2 - 30), "G");
        mvprintw(i, (COLS / 2 - 20), blank);
        mvprintw(i + 1, (COLS / 2 - 20), "A");
        mvprintw(i, (COLS / 2 - 10), blank);
        mvprintw(i + 1, (COLS / 2 - 10), "M");
        mvprintw(i, (COLS / 2), blank);
        mvprintw(i + 1, (COLS / 2), "E");

        mvprintw(j, (COLS / 2 - 30), blank);
        mvprintw(j - 1, (COLS / 2 - 30), "O");
        mvprintw(j, (COLS / 2 - 20), blank);
        mvprintw(j - 1, (COLS / 2 - 20), "V");
        mvprintw(j, (COLS / 2 - 10), blank);
        mvprintw(j - 1, (COLS / 2 - 10), "E");
        mvprintw(j, (COLS / 2), blank);
        mvprintw(j - 1, (COLS / 2), "R");
        napms(100);
        refresh();
    }
    sndoff();
    nocbreak(); 
    getch(); 
    attrset(A_NORMAL);
    refresh(); 
    flushinp(); 
    endwin(); 
    exit(0); 
}


int check_bounds_y(int y)
{
    if (y == YMIN) 
        return 1;
    if (y == (LINES - 2)) 
        return 2;
	
    return 0;
}


int check_bounds_x(int x)
{
    if (x == XMIN) 
        return 1;
    if (x == (COLS - 2)) 
        return 2;
	
    return 0;
}


void big_deal(void)
{
WINDOW *win = newwin(0, 0, 0, 0);
char *logname = getenv("LOGNAME");
char ending[] = "You are a true star dork.";
int fr = 0;

    wattrset(win, A_BOLD);
    box(win, ' ', ' ');
    mvwprintw(win, 4, (COLS / 2) - 3, "You Win %s", logname ? logname : " ");
 
    srand((unsigned)time(NULL));
    for (i = 0; i < sizeof(ending); i++) {
        fr = (rand() % 550) + 100;
        sndon(fr);
        mvwprintw(win, 12, i + ((COLS / 2) - 12), "%c", ending[i]);
        napms(200); 
        wrefresh(win);
    }
    sndoff();
    wgetch(win);
    wattrset(win, A_NORMAL);
    wrefresh(win);	
    flushinp(); 
    endwin(); 
    exit(0);
}   


static void catch_sigint(int signo)
{
    write(1, "Bye!\n", 5);
    sndoff();
    attrset(A_NORMAL);
    refresh();
    endwin();
    exit(0);
}
